# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: compas_timber_pb/generated/building_plan.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'compas_timber_pb/generated/building_plan.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from compas_pb.generated import mesh_pb2 as compas__pb_dot_generated_dot_mesh__pb2
from compas_pb.generated import box_pb2 as compas__pb_dot_generated_dot_box__pb2
from compas_pb.generated import cylinder_pb2 as compas__pb_dot_generated_dot_cylinder__pb2
from compas_pb.generated import sphere_pb2 as compas__pb_dot_generated_dot_sphere__pb2
from compas_timber_pb.generated import elements_pb2 as compas__timber__pb_dot_generated_dot_elements__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n.compas_timber_pb/generated/building_plan.proto\x12\x15\x63ompas_timber_pb.data\x1a\x1e\x63ompas_pb/generated/mesh.proto\x1a\x1d\x63ompas_pb/generated/box.proto\x1a\"compas_pb/generated/cylinder.proto\x1a compas_pb/generated/sphere.proto\x1a)compas_timber_pb/generated/elements.proto\"\x96\x01\n\x08StepData\x12\x0c\n\x04guid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x14\n\x0c\x65lementGuids\x18\x03 \x03(\t\x12\x13\n\x0b\x65lementHeld\x18\x04 \x03(\t\x12\x0f\n\x07isBuilt\x18\x05 \x01(\x08\x12\x11\n\tisPlanned\x18\x06 \x01(\x08\x12\r\n\x05\x61\x63tor\x18\x07 \x01(\x05\x12\x10\n\x08priority\x18\x08 \x01(\x05\"^\n\x10\x42uildingPlanData\x12\x0c\n\x04guid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12.\n\x05steps\x18\x03 \x03(\x0b\x32\x1f.compas_timber_pb.data.StepData\"y\n\x07\x45lement\x12/\n\x04\x62\x65\x61m\x18\x01 \x01(\x0b\x32\x1f.compas_timber_pb.data.BeamDataH\x00\x12\x35\n\x07\x65lement\x18\x02 \x01(\x0b\x32\".compas_timber_pb.data.ElementDataH\x00\x42\x06\n\x04kind\"\xe7\x01\n\x0f\x45lementGeometry\x12\x1a\n\x10\x66irebaseFilename\x18\x01 \x01(\tH\x00\x12(\n\x04mesh\x18\x02 \x01(\x0b\x32\x18.compas_pb.data.MeshDataH\x00\x12&\n\x03\x62ox\x18\x03 \x01(\x0b\x32\x17.compas_pb.data.BoxDataH\x00\x12\x30\n\x08\x63ylinder\x18\x04 \x01(\x0b\x32\x1c.compas_pb.data.CylinderDataH\x00\x12,\n\x06sphere\x18\x05 \x01(\x0b\x32\x1a.compas_pb.data.SphereDataH\x00\x42\x06\n\x04kind\"\xb5\x03\n\x1e\x42uildingPlanModelContainerData\x12\x35\n\x04plan\x18\x01 \x01(\x0b\x32\'.compas_timber_pb.data.BuildingPlanData\x12U\n\x08\x65lements\x18\x02 \x03(\x0b\x32\x43.compas_timber_pb.data.BuildingPlanModelContainerData.ElementsEntry\x12Y\n\ngeometries\x18\x03 \x03(\x0b\x32\x45.compas_timber_pb.data.BuildingPlanModelContainerData.GeometriesEntry\x1aO\n\rElementsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12-\n\x05value\x18\x02 \x01(\x0b\x32\x1e.compas_timber_pb.data.Element:\x02\x38\x01\x1aY\n\x0fGeometriesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x35\n\x05value\x18\x02 \x01(\x0b\x32&.compas_timber_pb.data.ElementGeometry:\x02\x38\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compas_timber_pb.generated.building_plan_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_BUILDINGPLANMODELCONTAINERDATA_ELEMENTSENTRY']._loaded_options = None
  _globals['_BUILDINGPLANMODELCONTAINERDATA_ELEMENTSENTRY']._serialized_options = b'8\001'
  _globals['_BUILDINGPLANMODELCONTAINERDATA_GEOMETRIESENTRY']._loaded_options = None
  _globals['_BUILDINGPLANMODELCONTAINERDATA_GEOMETRIESENTRY']._serialized_options = b'8\001'
  _globals['_STEPDATA']._serialized_start=250
  _globals['_STEPDATA']._serialized_end=400
  _globals['_BUILDINGPLANDATA']._serialized_start=402
  _globals['_BUILDINGPLANDATA']._serialized_end=496
  _globals['_ELEMENT']._serialized_start=498
  _globals['_ELEMENT']._serialized_end=619
  _globals['_ELEMENTGEOMETRY']._serialized_start=622
  _globals['_ELEMENTGEOMETRY']._serialized_end=853
  _globals['_BUILDINGPLANMODELCONTAINERDATA']._serialized_start=856
  _globals['_BUILDINGPLANMODELCONTAINERDATA']._serialized_end=1293
  _globals['_BUILDINGPLANMODELCONTAINERDATA_ELEMENTSENTRY']._serialized_start=1123
  _globals['_BUILDINGPLANMODELCONTAINERDATA_ELEMENTSENTRY']._serialized_end=1202
  _globals['_BUILDINGPLANMODELCONTAINERDATA_GEOMETRIESENTRY']._serialized_start=1204
  _globals['_BUILDINGPLANMODELCONTAINERDATA_GEOMETRIESENTRY']._serialized_end=1293
# @@protoc_insertion_point(module_scope)
