# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: compas_timber_pb/generated/processing.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'compas_timber_pb/generated/processing.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from compas_pb.generated import message_pb2 as compas__pb_dot_generated_dot_message__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+compas_timber_pb/generated/processing.proto\x12\x15\x63ompas_timber_pb.data\x1a!compas_pb/generated/message.proto\"\xbf\x01\n\x12\x42TLxProcessingData\x12\x0c\n\x04guid\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x45\n\x06params\x18\x03 \x03(\x0b\x32\x35.compas_timber_pb.data.BTLxProcessingData.ParamsEntry\x1a\x46\n\x0bParamsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12&\n\x05value\x18\x02 \x01(\x0b\x32\x17.compas_pb.data.AnyData:\x02\x38\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'compas_timber_pb.generated.processing_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_BTLXPROCESSINGDATA_PARAMSENTRY']._loaded_options = None
  _globals['_BTLXPROCESSINGDATA_PARAMSENTRY']._serialized_options = b'8\001'
  _globals['_BTLXPROCESSINGDATA']._serialized_start=106
  _globals['_BTLXPROCESSINGDATA']._serialized_end=297
  _globals['_BTLXPROCESSINGDATA_PARAMSENTRY']._serialized_start=227
  _globals['_BTLXPROCESSINGDATA_PARAMSENTRY']._serialized_end=297
# @@protoc_insertion_point(module_scope)
